package com.jpattern.orm;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.jpattern.orm.dialect.DerbyDialect;
import com.jpattern.orm.test.AutoIdTest;
import com.jpattern.orm.test.EmployeeTest;
import com.jpattern.orm.test.mapper.ClassMapperGeneratorTest;
import com.jpattern.orm.test.query.QueryExecutionTest;
import com.jpattern.orm.test.query.forupdate.QuerySelectForUpdateExecutionTest;
import com.jpattern.orm.test.transaction.EmployeeTransactionTest;

/**
 * 
 * @author Francesco Cina
 * 
 *         20/mag/2011
 */
public class ComplianceDERBYTests extends TestCase {
	/**
	 * Create the test case
	 * 
	 * @param testName
	 *            name of the test case
	 */
	public ComplianceDERBYTests(String testName) {
		super(testName);
	}

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		
		try {
			BaseTest.init(SessionProviderProxy.getSessionsProvider().getDerbySessionProvider(), 
					IJpOrmTestsConstants.DERBY_CREATE_SCRIPT, 
					IJpOrmTestsConstants.DERBY_DROP_SCRIPT,
					new DerbyDialect());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		
		TestSuite suite = new TestSuite("COMPLIANCE DERBY TEST SUITE FOR jPattern ORM");

		suite.addTestSuite(EmployeeTest.class);
		suite.addTestSuite(ClassMapperGeneratorTest.class);
		suite.addTestSuite(EmployeeTransactionTest.class);
		suite.addTestSuite(QueryExecutionTest.class);
		suite.addTestSuite(AutoIdTest.class);
		suite.addTestSuite(QuerySelectForUpdateExecutionTest.class);
		

		// There are problems with Derby's generated key logic, then transactions do not work. 
//		suite.addTestSuite(PeopleMultipleTest.class);
//		suite.addTestSuite(PeopleTest.class);
//		suite.addTestSuite(ScriptExecutorTest.class);
//		suite.addTestSuite(JdbcTemplatePeopleTest.class);
		
		return suite;
	}
}
