package com.jpattern.orm;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.jpattern.orm.dialect.HSQLDB2Dialect;
import com.jpattern.orm.test.AutoIdTest;
import com.jpattern.orm.test.EmployeeTest;
import com.jpattern.orm.test.PeopleMultipleTest;
import com.jpattern.orm.test.PeopleTest;
import com.jpattern.orm.test.mapper.ClassMapperGeneratorTest;
import com.jpattern.orm.test.query.QueryExecutionTest;
import com.jpattern.orm.test.script.ScriptExecutorTest;
import com.jpattern.orm.test.transaction.EmployeeTransactionTest;
import com.jpattern.orm.test.transaction.JdbcTemplatePeopleTest;

/**
 * 
 * @author Francesco Cina
 * 
 *         20/mag/2011
 */
public class ComplianceHSQLDBTests extends TestCase {
	/**
	 * Create the test case
	 * 
	 * @param testName
	 *            name of the test case
	 */
	public ComplianceHSQLDBTests(String testName) {
		super(testName);
	}

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		
		try {
			BaseTest.init(SessionProviderProxy.getSessionsProvider().getHSQLDBSessionProvider(), 
					IJpOrmTestsConstants.HSQLDB_CREATE_SCRIPT, 
					IJpOrmTestsConstants.HSQLDB_DROP_SCRIPT,
					new HSQLDB2Dialect());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		
		TestSuite suite = new TestSuite("COMPLIANCE HSQLDB TEST SUITE FOR jPattern ORM");

		suite.addTestSuite(EmployeeTest.class);
		suite.addTestSuite(ClassMapperGeneratorTest.class);
		suite.addTestSuite(PeopleTest.class);
		suite.addTestSuite(EmployeeTransactionTest.class);
		suite.addTestSuite(QueryExecutionTest.class);
		suite.addTestSuite(PeopleMultipleTest.class);
		suite.addTestSuite(ScriptExecutorTest.class);
		suite.addTestSuite(AutoIdTest.class);
		suite.addTestSuite(JdbcTemplatePeopleTest.class);
		
		//HSQLDB doesn't support SELECT...FOR UPDATE
//		suite.addTestSuite(QuerySelectForUpdateExecutionTest.class);
		return suite;
	}
}
