package com.jpattern.orm.test.crud;


import com.jpattern.orm.BaseTest;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.test.domain.Employee;
import com.jpattern.orm.test.domain.People;
import com.jpattern.orm.test.domain.Zoo_People;
import com.jpattern.orm.crud.IOrmCRUDQuery;
import com.jpattern.orm.crud.IOrmCRUDQueryGenerator;
import com.jpattern.orm.crud.OrmCRUDQueryGenerator;
import com.jpattern.orm.dialect.DefaultDialect;
import com.jpattern.orm.mapper.ClassTableMapBuilder;
import com.jpattern.orm.mapper.ClassMapperBuilder;
import com.jpattern.orm.mapper.IClassMapper;
import com.jpattern.orm.mapper.IClassMapperBuilder;
import com.jpattern.orm.mapper.TableMap;

/**
 * 
 * @author Francesco Cina
 *
 * 01/giu/2011
 */
public class OrmCRUDQueryGeneratorTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testOrmProvider() throws Exception {
		ISessionProvider connectionProvider = getSessionProvider();
		ormCrudWithSequence(connectionProvider);
		ormCrud(connectionProvider);
		ormCrudWithSchema(connectionProvider);
		ormCrudSelectFromClause(connectionProvider);
	}
	
	public void ormCrudWithSequence(ISessionProvider sessionProvider) throws Exception {
		Class<People> clazz = People.class;
		TableMap tableMap = new TableMap();
		new ClassTableMapBuilder(clazz, tableMap).updateTableMap();
		sessionProvider.updateTableMap(tableMap , true);
		IClassMapperBuilder<People> classMapperGenerator = new ClassMapperBuilder<People>(clazz, tableMap);
		IClassMapper<People> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		IOrmCRUDQueryGenerator<People> ormGenerator = new OrmCRUDQueryGenerator<People>(new DefaultDialect(), classMapper);
		IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);
		
		System.out.println(ormCrudQuery.getLoadQuery());
		System.out.println(ormCrudQuery.getUpdateQuery());
		System.out.println(ormCrudQuery.getDeleteQuery());
		System.out.println(ormCrudQuery.getSaveQuery());
	}
	
	public void ormCrud(ISessionProvider sessionProvider) throws Exception {
		Class<Employee> clazz = Employee.class;
		TableMap tableMap = new TableMap();
		new ClassTableMapBuilder(clazz, tableMap).updateTableMap();
		sessionProvider.updateTableMap(tableMap , true);
		IClassMapperBuilder<Employee> classMapperGenerator = new ClassMapperBuilder<Employee>(clazz, tableMap);
		IClassMapper<Employee> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		IOrmCRUDQueryGenerator<Employee> ormGenerator = new OrmCRUDQueryGenerator<Employee>(new DefaultDialect(), classMapper);
		IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);
		
		System.out.println(ormCrudQuery.getLoadQuery());
		System.out.println(ormCrudQuery.getUpdateQuery());
		System.out.println(ormCrudQuery.getDeleteQuery());
		System.out.println(ormCrudQuery.getSaveQuery());
	}
	
	public void ormCrudWithSchema(ISessionProvider sessionProvider) throws Exception {
		Class<Zoo_People> clazz = Zoo_People.class;
		TableMap tableMap = new TableMap();
		new ClassTableMapBuilder(clazz, tableMap).updateTableMap();
		sessionProvider.updateTableMap(tableMap , true);
		IClassMapperBuilder<Zoo_People> classMapperGenerator = new ClassMapperBuilder<Zoo_People>(clazz, tableMap);
		IClassMapper<Zoo_People> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		IOrmCRUDQueryGenerator<Zoo_People> ormGenerator = new OrmCRUDQueryGenerator<Zoo_People>(new DefaultDialect(), classMapper);
		IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);
		
		System.out.println(ormCrudQuery.getLoadQuery());
		System.out.println(ormCrudQuery.getUpdateQuery());
		System.out.println(ormCrudQuery.getDeleteQuery());
		System.out.println(ormCrudQuery.getSaveQuery());
	}
	
	public void ormCrudSelectFromClause(ISessionProvider sessionProvider) throws Exception {
		Class<Zoo_People> clazz = Zoo_People.class;
		TableMap tableMap = new TableMap();
		new ClassTableMapBuilder(clazz, tableMap).updateTableMap();
		sessionProvider.updateTableMap(tableMap , true);
		IClassMapperBuilder<Zoo_People> classMapperGenerator = new ClassMapperBuilder<Zoo_People>(clazz, tableMap);
		IClassMapper<Zoo_People> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		IOrmCRUDQueryGenerator<Zoo_People> ormGenerator = new OrmCRUDQueryGenerator<Zoo_People>(new DefaultDialect(), classMapper);
		IOrmCRUDQuery ormCrudQuery = ormGenerator.generate();
		assertNotNull(ormCrudQuery);
		
		System.out.println(ormCrudQuery.getBaseSelectClause());
		System.out.println(ormCrudQuery.getBaseSelectClause("alias."));
		System.out.println(ormCrudQuery.getBaseFromClause());
		
		assertEquals("ZOO.PEOPLE.FIRSTCLOB, ZOO.PEOPLE.ID, ZOO.PEOPLE.SECONDBLOB, ZOO.PEOPLE.BIRTHDATE, ZOO.PEOPLE.LASTNAME, ZOO.PEOPLE.FIRSTNAME, ZOO.PEOPLE.FIRSTBLOB, ZOO.PEOPLE.DEATHDATE", ormCrudQuery.getBaseSelectClause());
		assertEquals("alias.FIRSTCLOB, alias.ID, alias.SECONDBLOB, alias.BIRTHDATE, alias.LASTNAME, alias.FIRSTNAME, alias.FIRSTBLOB, alias.DEATHDATE", ormCrudQuery.getBaseSelectClause("alias."));
		assertEquals("ZOO.PEOPLE", ormCrudQuery.getBaseFromClause());

	}
}
