package com.jpattern.orm.test.h2;

import java.io.InputStream;
import java.io.Reader;
import java.util.Date;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.transaction.ITransaction;
import com.jpattern.orm.test.domain.People;
import com.jpattern.orm.util.OrmUtil;

/**
 * 
 * @author Francesco Cina
 *
 * 20/mag/2011
 */
public class People2Test extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	
	public void testCrudPeople() throws Exception {
		IJPOrm jpOrm =getJPOrm();
		
		jpOrm.register(People.class);
		
        long id = new Date().getTime();
        
        String text1 = "BINARY STRING TEST 1 " + id;
		InputStream is = OrmUtil.convertStringToStream(text1, OrmUtil.UTF8);

        String text2 = "BINARY STRING TEST 2 " + id;
		Reader reader = OrmUtil.convertStringToReader(text2);
		
		People people = new People();
		people.setId( id );
		people.setFirstname( "people" );
		people.setLastname("Wizard");
		people.setFirstblob(is);
		people.setFirstclob(reader);
		
		// CREATE
        ISession conn = jpOrm.session();
        ITransaction tx = conn.getTransaction();
        conn.save(people);
        tx.commit();
        
        reader.close();
        is.close();
        
        System.out.println("People saved with id: " + people.getId());
        assertFalse( id == people.getId() );
        id = people.getId();
        
        // LOAD
        tx = conn.getTransaction();
        People peopleLoad1 = conn.find(People.class, new Object[]{id});
        assertNotNull(peopleLoad1);
        assertEquals( people.getId(), peopleLoad1.getId() );
        assertEquals( people.getFirstname(), peopleLoad1.getFirstname() );
        assertEquals( people.getLastname(), peopleLoad1.getLastname() );
        
        String retrieved1 = OrmUtil.convertStreamToString( peopleLoad1.getFirstblob(), OrmUtil.UTF8, false);
        System.out.println("Retrieved1 String " + retrieved1);
        assertEquals( text1 , retrieved1 );
        
        String retrieved2 = OrmUtil.convertReaderToString( peopleLoad1.getFirstclob() , false);
        System.out.println("Retrieved2 String " + retrieved2);
        assertEquals( text2 , retrieved2 );
        
        
        //UPDATE
        peopleLoad1.setFirstname("Wizard name");
        conn.update(peopleLoad1);
        tx.commit();
        
        // LOAD
        tx = conn.getTransaction();
        People peopleLoad2 = conn.find(People.class, new Object[]{id});
        assertNotNull(peopleLoad2);
        assertEquals( peopleLoad1.getId(), peopleLoad2.getId() );
        assertEquals( peopleLoad1.getFirstname(), peopleLoad2.getFirstname() );
        assertEquals( peopleLoad1.getLastname(), peopleLoad2.getLastname() );
        
        //DELETE
        conn.delete(peopleLoad2);
        People peopleLoad3 = conn.find(People.class, new Object[]{id});
        assertNull(peopleLoad3);
        tx.commit();
        
	}

}
