package com.jpattern.orm.test.mapper;

import java.util.Arrays;
import java.util.Date;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.mapper.ClassMapperBuilder;
import com.jpattern.orm.mapper.ClassTableMapBuilder;
import com.jpattern.orm.mapper.IClassMapper;
import com.jpattern.orm.mapper.IClassMapperBuilder;
import com.jpattern.orm.mapper.IColumn;
import com.jpattern.orm.mapper.NullColumn;
import com.jpattern.orm.mapper.TableMap;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.test.domain.Blobclob_ByteArray;
import com.jpattern.orm.test.domain.Blobclob_Stream;
import com.jpattern.orm.test.domain.Employee;

/**
 * 
 * @author Francesco Cina
 *
 * 01/giu/2011
 */
public class ClassMapperGeneratorTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testGenerator() throws Exception {
		ISessionProvider connectionProvider = getSessionProvider();
		classMapperGenerator1(connectionProvider);
		classMapperGenerator2(connectionProvider);
		classMapperGenerator3(connectionProvider);
	}

	public void classMapperGenerator1(ISessionProvider sessionProvider) throws Exception {
		Class<Employee> clazz = Employee.class;
		TableMap classDBMap = new TableMap();
		new ClassTableMapBuilder(clazz, classDBMap).updateTableMap();
		sessionProvider.updateTableMap(classDBMap , true);
		IClassMapperBuilder<Employee> classMapperGenerator = new ClassMapperBuilder<Employee>(clazz, classDBMap);
		
		IClassMapper<Employee> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		assertEquals("" , classMapper.getTableMap().getSchemaName());
		assertEquals("EMPLOYEE", classMapper.getTableMap().getTableNameWithSchema());
		assertEquals(5 , classMapper.getAllColumnJavaNames().length );
		assertEquals(1 , classMapper.getPrimaryKeyColumnJavaNames().length );
		assertEquals(4 , classMapper.getNotPrimaryKeyColumnJavaNames().length );
		
		assertEquals( clazz , classMapper.getMappedClass());
		
		assertTrue( classMapper.getColumnWithJavaName("" + new Date().getTime()) instanceof NullColumn );
		
		for (String col : classMapper.getAllColumnJavaNames()) {
			System.out.println("Search column " + col);
			assertFalse( classMapper.getColumnWithJavaName(col) instanceof NullColumn );
		}
		
		for (String col : classMapper.getPrimaryKeyColumnJavaNames()) {
			System.out.println("Search column " + col);
			IColumn column = classMapper.getColumnWithJavaName(col);
			assertFalse( column instanceof NullColumn );
			assertTrue( column.isIdentifying() );
		}
		
		for (String col : classMapper.getNotPrimaryKeyColumnJavaNames()) {
			System.out.println("Search column " + col);
			IColumn column = classMapper.getColumnWithJavaName(col);
			assertFalse( column instanceof NullColumn );
			assertFalse( column.isIdentifying() );
		}
		
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "id" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "age" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "name" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "surname" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "employeeNumber" ) );
		
		assertTrue( Arrays.asList(classMapper.getPrimaryKeyColumnJavaNames()).contains( "id" ) );
		
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "age" ) );
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "name" ) );
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "surname" ) );
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "employeeNumber" ) );
		
		assertEquals( "ID" ,  classMapper.getColumnWithJavaName("id").getName() );
		assertEquals( "AGE" ,  classMapper.getColumnWithJavaName("age").getName() );
		assertEquals( "NAME" ,  classMapper.getColumnWithJavaName("name").getName() );
		assertEquals( "SURNAME" ,  classMapper.getColumnWithJavaName("surname").getName() );
		assertEquals( "EMPLOYEE_NUMBER" ,  classMapper.getColumnWithJavaName("employeeNumber").getName() );
		
	}
	
	public void classMapperGenerator2(ISessionProvider connectionProvider) throws Exception {
		
		Class<Blobclob_Stream> clazz = Blobclob_Stream.class;
		TableMap classDBMap = new TableMap();
		new ClassTableMapBuilder(clazz, classDBMap).updateTableMap();
		connectionProvider.updateTableMap(classDBMap , true);
		IClassMapperBuilder<Blobclob_Stream> classMapperGenerator = new ClassMapperBuilder<Blobclob_Stream>(clazz, classDBMap);
		
		IClassMapper<Blobclob_Stream> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		assertEquals("" , classMapper.getTableMap().getSchemaName());
		assertEquals("BLOBCLOB", classMapper.getTableMap().getTableNameWithSchema());
		assertEquals(3 , classMapper.getAllColumnJavaNames().length );
		assertEquals(1 , classMapper.getPrimaryKeyColumnJavaNames().length );
		assertEquals(2 , classMapper.getNotPrimaryKeyColumnJavaNames().length );
		
		assertEquals( clazz , classMapper.getMappedClass());
		
		assertTrue( classMapper.getColumnWithJavaName("" + new Date().getTime()) instanceof NullColumn );
		
		for (String col : classMapper.getAllColumnJavaNames()) {
			System.out.println("Search column " + col);
			assertFalse( classMapper.getColumnWithJavaName(col) instanceof NullColumn );
		}
		
		for (String col : classMapper.getPrimaryKeyColumnJavaNames()) {
			System.out.println("Search column " + col);
			IColumn column = classMapper.getColumnWithJavaName(col);
			assertFalse( column instanceof NullColumn );
			assertTrue( column.isIdentifying() );
		}
		
		for (String col : classMapper.getNotPrimaryKeyColumnJavaNames()) {
			System.out.println("Search column " + col);
			IColumn column = classMapper.getColumnWithJavaName(col);
			assertFalse( column instanceof NullColumn );
			assertFalse( column.isIdentifying() );
		}
		
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "id" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "blobInputStream" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "clob" ) );
		
		assertTrue( Arrays.asList(classMapper.getPrimaryKeyColumnJavaNames()).contains( "id" ) );
		
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "blobInputStream" ) );
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "clob" ) );
		
		assertEquals( "ID" ,  classMapper.getColumnWithJavaName("id").getName() );
		assertEquals( "BLOB" ,  classMapper.getColumnWithJavaName("blobInputStream").getName() );
		assertEquals( "CLOB" ,  classMapper.getColumnWithJavaName("clob").getName() );
		
	}
	
	public void classMapperGenerator3(ISessionProvider connectionProvider) throws Exception {
		
		Class<Blobclob_ByteArray> clazz = Blobclob_ByteArray.class;
		TableMap classDBMap = new TableMap();
		new ClassTableMapBuilder(clazz, classDBMap).updateTableMap();
		connectionProvider.updateTableMap(classDBMap , true);
		IClassMapperBuilder<Blobclob_ByteArray> classMapperGenerator = new ClassMapperBuilder<Blobclob_ByteArray>(clazz, classDBMap);
		
		IClassMapper<Blobclob_ByteArray> classMapper = classMapperGenerator.generate();
		assertNotNull(classMapper);
		
		assertEquals("" , classMapper.getTableMap().getSchemaName());
		assertEquals("BLOBCLOB", classMapper.getTableMap().getTableNameWithSchema());
		assertEquals(3 , classMapper.getAllColumnJavaNames().length );
		assertEquals(1 , classMapper.getPrimaryKeyColumnJavaNames().length );
		assertEquals(2 , classMapper.getNotPrimaryKeyColumnJavaNames().length );
		assertEquals(1 , classMapper.getAllGeneratedColumnJavaNames().length );
		assertEquals(1 , classMapper.getAllGeneratedColumnDBNames().length );
		
		assertEquals( clazz , classMapper.getMappedClass());
		
		assertTrue( classMapper.getColumnWithJavaName("" + new Date().getTime()) instanceof NullColumn );
		
		for (String col : classMapper.getAllColumnJavaNames()) {
			System.out.println("Search column " + col);
			assertFalse( classMapper.getColumnWithJavaName(col) instanceof NullColumn );
		}
		
		for (String col : classMapper.getPrimaryKeyColumnJavaNames()) {
			System.out.println("Search column " + col);
			IColumn column = classMapper.getColumnWithJavaName(col);
			assertFalse( column instanceof NullColumn );
			assertTrue( column.isIdentifying() );
		}
		
		for (String col : classMapper.getNotPrimaryKeyColumnJavaNames()) {
			System.out.println("Search column " + col);
			IColumn column = classMapper.getColumnWithJavaName(col);
			assertFalse( column instanceof NullColumn );
			assertFalse( column.isIdentifying() );
		}
		
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "index" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "blob" ) );
		assertTrue( Arrays.asList(classMapper.getAllColumnJavaNames()).contains( "clob" ) );
		
		assertTrue( Arrays.asList(classMapper.getAllGeneratedColumnJavaNames()).contains( "index" ) );
		
		assertTrue( Arrays.asList(classMapper.getAllGeneratedColumnDBNames()).contains( "ID" ) );
		
		assertTrue( Arrays.asList(classMapper.getPrimaryKeyColumnJavaNames()).contains( "index" ) );
		
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "blob" ) );
		assertTrue( Arrays.asList(classMapper.getNotPrimaryKeyColumnJavaNames()).contains( "clob" ) );
		
		assertEquals( "ID" ,  classMapper.getColumnWithJavaName("index").getName() );
		assertEquals( "BLOB" ,  classMapper.getColumnWithJavaName("blob").getName() );
		assertEquals( "CLOB" ,  classMapper.getColumnWithJavaName("clob").getName() );
		
	}
	
}
