package com.jpattern.orm.test.query;

import java.util.Date;
import java.util.List;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.JPOrm;
import com.jpattern.orm.query.IOrmQuery;
import com.jpattern.orm.query.expression.LeExpressionElement;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.transaction.ITransaction;
import com.jpattern.orm.test.domain.Employee;
import com.jpattern.orm.test.domain.Zoo_People;

/**
 * 
 * @author Francesco Cina
 *
 * 23/giu/2011
 */
public class QueryExecutionTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testCrupPeople() throws Exception {
		ISessionProvider sessionProvider = getSessionProvider();
		query1( sessionProvider );
		query2( sessionProvider );
		query3( sessionProvider );
	}
	
	public void query1(ISessionProvider sessionProvider) throws Exception {
		IJPOrm jpOrm = new JPOrm("",sessionProvider);
		jpOrm.register(Employee.class);
		jpOrm.register(Zoo_People.class);

		ISession session =  jpOrm.session();
		Employee employee = createEmployee(jpOrm);
		
		IOrmQuery<Employee> query = session.findQuery(Employee.class);
		System.out.println(query.renderSql());
		System.out.println(query.getGeneratedRowCountSql());
		String expectedSql = "SELECT Employee.ID, Employee.NAME, Employee.AGE, Employee.SURNAME, Employee.EMPLOYEE_NUMBER FROM EMPLOYEE Employee ";
		assertEquals(expectedSql , query.renderSql());
		
		List<Employee> employeeList = query.findList();
		assertNotNull( employeeList );
		
		long countRowQueryResult = query.findRowCount();
		
		System.out.println("found employees: " + employeeList.size());
		System.out.println("count row query result: " + countRowQueryResult);
		assertTrue( employeeList.size()>0 );
		assertEquals( employeeList.size(), countRowQueryResult );
		
		deleteEmployee(jpOrm, employee);
	}
	
	public void query2(ISessionProvider sessionProvider) throws Exception {
		IJPOrm jpOrm = new JPOrm("",sessionProvider);
		jpOrm.register(Employee.class);
		jpOrm.register(Zoo_People.class);

		ISession session =  jpOrm.session();
		Employee employee = createEmployee(jpOrm);
		
		int maxRows = 4;
		IOrmQuery<Employee> query = session.findQuery(Employee.class, Zoo_People.class);
		query.setMaxRows(maxRows);
		query.where().not( new LeExpressionElement("Employee.id", "0") );
		query.where().ilike("Zoo_People.firstname", "%");
		System.out.println(query.renderSql());
		String expectedSql = "SELECT Employee.ID, Employee.NAME, Employee.AGE, Employee.SURNAME, Employee.EMPLOYEE_NUMBER FROM EMPLOYEE Employee , ZOO.PEOPLE Zoo_People WHERE NOT ( Employee.ID <= ? ) AND LOWER(Zoo_People.FIRSTNAME) like LOWER(?) ";
		assertEquals(expectedSql , query.renderSql());
		
		List<Employee> employeeList = query.findList();
		assertNotNull( employeeList );
		
		System.out.println("found employees: " + employeeList.size());
		assertTrue( employeeList.size()<=maxRows );
		
		deleteEmployee(jpOrm, employee);
	}
	
	public void query3(ISessionProvider sessionProvider) throws Exception {
		IJPOrm jpOrm = new JPOrm("",sessionProvider);
		jpOrm.register(Employee.class);
		jpOrm.register(Zoo_People.class);

		ISession session =  jpOrm.session();
		Employee employee = createEmployee(jpOrm);
		
		int maxRows = 4;
		IOrmQuery<Employee> query = session.findQuery(Employee.class, "e");
		query.setMaxRows(maxRows);
		query.where().ge("e.id", "0");
		System.out.println(query.renderSql());
		String expectedSql = "SELECT e.ID, e.NAME, e.AGE, e.SURNAME, e.EMPLOYEE_NUMBER FROM EMPLOYEE e WHERE e.ID >= ? ";
		assertEquals(expectedSql , query.renderSql());
		
		List<Employee> employeeList = query.findList();
		assertNotNull( employeeList );
		
		System.out.println("found employees: " + employeeList.size());
		assertTrue( employeeList.size()>0 );
		assertTrue( employeeList.size()<=maxRows );
		
		deleteEmployee(jpOrm, employee);
	}
	
	private Employee createEmployee(IJPOrm jpOrm) {
		ISession ormSession = jpOrm.session();
		ITransaction tx = ormSession.getTransaction();
        long id = new Date().getTime();
		Employee employee = new Employee();
		employee.setId( id );
		employee.setAge( 44 );
		employee.setEmployeeNumber( ("" + id).substring(0, 9) );
		employee.setName("Wizard");
		employee.setSurname("Cina");
		ormSession.save(employee);
		tx.commit();
		return employee;
	}
	
	private void deleteEmployee(IJPOrm jpOrm, Employee employee) {
		ISession ormSession = jpOrm.session();
		ITransaction tx = ormSession.getTransaction();
		ormSession.delete(employee);
		tx.commit();
	}
	
}
