package com.jpattern.orm.test.session;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.session.IGeneratedKeyReader;
import com.jpattern.orm.session.IResultSetReader;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.session.ISessionSqlPerformer;
import com.jpattern.orm.session.ISqlPerformer;
import com.jpattern.orm.transaction.ITransaction;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 */
public class SqlExecutorsTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testExecuteAll() throws Exception {
		IJPOrm jpOrm =getJPOrm();
		
        ISession session = jpOrm.session();
        ISqlPerformer sqlExecutor = ((ISessionSqlPerformer)session).sqlPerformer();
        
        ITransaction tx = session.getTransaction();
        List<Long> ids = sqlExecutorInsert( sqlExecutor );
        tx.commit();
		
		checkExistAll(ids , sqlExecutor , true);
		
		tx = session.getTransaction();
		sqlExecutorDelete( ids, sqlExecutor );
		tx.commit();

		checkExistAll(ids , sqlExecutor , false);
		
	}
	

	private void sqlExecutorDelete(List<Long> ids, ISqlPerformer sqlExecutor) {
		String sql = "delete from people where id = ?";
		
		List<Object[]> args = new ArrayList<Object[]>();
		
		for (Long id : ids) {
			args.add(new Object[]{id});
		}
		
		sqlExecutor.batchUpdate(sql, args);
		
	}

	private List<Long> sqlExecutorInsert(ISqlPerformer sqlExec) {
		final List<Long> results = new ArrayList<Long>();
		
		long idMain = new Date().getTime();
		
		long id1 = idMain++;
		results.add(id1);
		String sqlFixed = "insert into people (id, firstname, lastname) values ( " + id1 + " , 'fixed name' , 'fixed surname' )";
		assertEquals( 1 , sqlExec.update(sqlFixed));
		
		String sql1 = "insert into people (id, firstname, lastname) values ( ? , ? , ? )";
		long id2 = idMain++;
		results.add(id2);
		assertEquals( 1 , sqlExec.update(sql1, new Object[]{ id2, "name-" + id2 , "surname-" + id2 }));
		
		List<Object[]> args = new ArrayList<Object[]>();
		long id3 = idMain++;
		long id4 = idMain++;
		long id5 = idMain++;
		results.add(id3);
		results.add(id4);
		results.add(id5);
		args.add( new Object[]{ id3, "name-" + id3 , "batchUpdate(sql1, args) " + id3 } );
		args.add( new Object[]{ id4, "name-" + id4 , "batchUpdate(sql1, args) " + id4 } );
		args.add( new Object[]{ id5, "name-" + id5 , "batchUpdate(sql1, args) " + id5 } );
		assertEquals( 3 , sqlExec.batchUpdate(sql1, args).length );
		
		List<String> sqlsFixed = new ArrayList<String>();
		long id6 = idMain++;
		long id7 = idMain++;
		results.add(id6);
		results.add(id7);
		sqlsFixed.add("insert into people (id, firstname, lastname) values ( " + id6 + " , 'batchUpdate(sqlsFixed)' , '1' )");
		sqlsFixed.add("insert into people (id, firstname, lastname) values ( " + id7 + " , 'batchUpdate(sqlsFixed)' , '2' )");
		assertEquals( 2 , sqlExec.batchUpdate(sqlsFixed).length );
		
		String sqlKeyExtractor = "insert into people (id, firstname, lastname) values ( SEQ_PEOPLE.nextval , ? , ? )";
		IGeneratedKeyReader generatedKeyExtractor = new IGeneratedKeyReader() {
			/**
			 * 
			 */
			

			@Override
			public void read(ResultSet generatedKeyResultSet) throws SQLException {
				generatedKeyResultSet.next();
				long gk = generatedKeyResultSet.getLong(1);
				System.out.println("Generated key: " + gk);
				results.add(gk);
			}

			@Override
			public String[] generatedColumnNames() {
				return new String[]{"ID"};
			}
		};
		assertEquals( 1 , sqlExec.update(sqlKeyExtractor, generatedKeyExtractor, new Object[]{ "sqlExec.update(sqlKeyExtractor, generatedKeyExtractor, args", "1"}) );
		
		return results;
	}
	
	private void checkExistAll(List<Long> peopleIds, ISqlPerformer sqlExecutor, boolean exist) {
		String sql = "select * from people where id in ( ";
		for (int i=0; i<peopleIds.size()-1; i++) {
			sql += "?, ";
		}
		sql += "? ) ";

		IResultSetReader<List<Long>> rse = new IResultSetReader<List<Long>>() {
			@Override
			public List<Long> read(ResultSet resultSet) throws SQLException {
				List<Long> result = new ArrayList<Long>();
				while (resultSet.next()) {
					result.add( resultSet.getLong("ID") );
				}
				return result;
			}
		};
		
		List<Long> result = sqlExecutor.query(sql, rse, peopleIds.toArray());
		for (Long id : peopleIds) {
			System.out.println("Check id: " + id + " exists? " + result.contains(id) );
			assertEquals( exist,  result.contains(id) );
		}
	}
}
