package com.jpattern.orm.test.transaction;

import java.util.Date;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.transaction.ITransaction;
import com.jpattern.orm.test.domain.People;

/**
 * 
 * @author Francesco Cina
 *
 * 20/mag/2011
 */
public class JdbcTemplatePeopleTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testJdbcTemplateTransaction1() throws Exception {
		IJPOrm jpOrm = getJPOrm();
		jpOrm.register(People.class);
		
		ITransaction tx = jpOrm.session().getTransaction();
		long id = create( jpOrm );
		tx.commit();
		
		tx = jpOrm.session().getTransaction();
		People loaded = load(jpOrm, id);
		tx.commit();
		assertNotNull(loaded);
		
		tx = jpOrm.session().getTransaction();
		delete(jpOrm, loaded);
		tx.rollback();
		
		tx = jpOrm.session().getTransaction();
		loaded = load(jpOrm, id);
		tx.commit();
		assertNotNull(loaded);
		
		tx = jpOrm.session().getTransaction();
		delete(jpOrm, loaded);
		tx.setRollbackOnly();
		tx.commit();
		
		tx = jpOrm.session().getTransaction();
		loaded = load(jpOrm, id);
		tx.commit();
		assertNotNull(loaded);
		
		tx = jpOrm.session().getTransaction();
		delete(jpOrm, loaded);
		tx.commit();
		
		tx = jpOrm.session().getTransaction();
		loaded = load(jpOrm, id);
		tx.commit();
		assertNull(loaded);
	}
	

	private long create(IJPOrm jpOrm) throws Exception {
		
        long id = new Date().getTime();
		People people = new People();
		people.setId( id );
		people.setFirstname( "people" );
		people.setLastname("Wizard");
		
		// CREATE
        ISession conn = jpOrm.session();
        conn.save(people);
        
        System.out.println("People saved with id: " + people.getId());
        assertFalse( id == people.getId() );
        return people.getId();
        
	}
	
	private People load(IJPOrm jpOrm, long id) throws Exception {
        // LOAD
		ISession conn = jpOrm.session();
        People peopleLoad1 = conn.find(People.class, new Object[]{id});
        return peopleLoad1;
	}
	
//	private void update(IJPOrm jpOrm, People people) throws Exception {
//        //UPDATE
//		people.setFirstname("Wizard name");
//		ISession conn = jpOrm.session();
//        conn.update(people);
//	}
		
    private void delete(IJPOrm jpOrm, People people) throws Exception {
        //DELETE
    	ISession conn = jpOrm.session();
        conn.delete(people);
	}

}
